.. highlight:: shell

============
Contributing
============

Contributions are welcome, and they are greatly appreciated! Every little bit
helps, and credit will always be given.

You can contribute in many ways:

Types of Contributions
----------------------

Report Bugs
~~~~~~~~~~~

Report bugs at https://github.com/release-depot/toolchest/issues.

If you are reporting a bug, please include:

* Your operating system name and version.
* Any details about your local setup that might be helpful in troubleshooting.
* Detailed steps to reproduce the bug.

Fix Bugs
~~~~~~~~

Look through the GitHub issues for bugs. Anything tagged with "bug" and "help
wanted" is open to whoever wants to implement it.

Implement Features
~~~~~~~~~~~~~~~~~~

Look through the GitHub issues for features. Anything tagged with "enhancement"
and "help wanted" is open to whoever wants to implement it.

Write Documentation
~~~~~~~~~~~~~~~~~~~

toolchest could always use more documentation, whether as part of the
official toolchest docs, in docstrings, or even on the web in blog posts,
articles, and such.

Submit Feedback
~~~~~~~~~~~~~~~

The best way to send feedback is to file an issue at https://github.com/release-depot/toolchest/issues.

If you are proposing a feature:

* Explain in detail how it would work.
* Keep the scope as narrow as possible, to make it easier to implement.
* Remember that this is a volunteer-driven project, and that contributions
  are welcome :)

Get Started!
------------

Ready to contribute? Here's how to set up `toolchest` for local development.

1. Fork the `toolchest` repo on GitHub.
2. Clone your fork locally::

    $ git clone git@github.com:your_name_here/toolchest.git

3. Install your local copy into a virtualenv. Assuming you have
   virtualenvwrapper installed, this is how you set up your fork for local
   development::

    $ mkvirtualenv toolchest
    $ cd toolchest/
    $ python setup.py develop

3a. Alternatively, use pipenv via the Makefile to setup a new virtualenv.
    Pipenv will create a new virtualenv with packages at the correct versions.
    Assuming you have pipenv installed, this is how you set up your fork for
    local development::

    $ make dev
    $ pipenv shell

4. Create a branch for local development::

    $ git checkout -b name-of-your-bugfix-or-feature

   Now you can make your changes locally.

5. When you're done making changes, check that your changes pass flake8 and the
   tests, including testing other Python versions with tox::

    $ make lint
    $ make test

6. Commit your changes and push your branch to GitHub::

    $ git add .
    $ git commit -m "Your detailed description of your changes."
    $ git push origin name-of-your-bugfix-or-feature

7. Submit a pull request through the GitHub website.

Pull Request Guidelines
-----------------------

Before you submit a pull request, check that it meets these guidelines:

1. The pull request should include tests and be signed.
2. If the pull request adds functionality, the docs should be updated. Put
   your new functionality into a function with a docstring, and add the
   feature to the list in README.rst.
3. The pull request should work for Python 3.6-3.9. All pull requests get run
   against our CI using github actions, and if your patch fails any tests, you
   will get an email notification from github, so it can be corrected.
4. If you are adding a submodule, do the following:

   a. Make sure that there is an :code:`__init__.py`` in the same folder as the submodule.
   b. In :code:`docs/source/` add a file named :code:`toolchest.<submodule_path>.rst` and
      make it the same format as the peer files there.
   c. You may also need to add the submodule to :code:`docs/source/toolchest.rst` or
      whatever the parent module is. It should be added in the same format as the other
      modules in the file.

Tips
----


Deploying
---------

A reminder for the maintainers on how to deploy.
Make sure all your changes are committed (including an entry in HISTORY.rst).
Then run::

$ bumpversion patch # possible: major / minor / patch
$ git push
$ git push --tags

Github Actions will then deploy to PyPI if tests pass.
